@@ -15,9 +15,9 @@ pub struct ArgsKwargs {
 
 impl ArgsKwargs {
     fn eq(&self, py: Python, other: &Self) -> PyResult<bool> {
-        if self.args.as_ref(py).eq(other.args.as_ref(py))? {
+        if self.args.bind(py).eq(other.args.bind(py))? {
             match (&self.kwargs, &other.kwargs) {
-                (Some(d1), Some(d2)) => d1.as_ref(py).eq(d2.as_ref(py)),
+                (Some(d1), Some(d2)) => d1.bind(py).eq(d2.bind(py)),
                 (None, None) => Ok(true),
                 _ => Ok(false),
             }
@@ -30,11 +30,11 @@ impl ArgsKwargs {
 #[pymethods]
 impl ArgsKwargs {
     #[new]
-    fn py_new(py: Python, args: &PyTuple, kwargs: Option<&PyDict>) -> Self {
+    fn py_new(py: Python, args: &Bound<'_, PyTuple>, kwargs: Option<&Bound<'_, PyDict>>) -> Self {
         Self {
             args: args.into_py(py),
             kwargs: match kwargs {
-                Some(d) if !d.is_empty() => Some(d.into_py(py)),
+                Some(d) if !d.is_empty() => Some(d.to_owned().unbind()),
                 _ => None,
             },
         }
@@ -55,9 +55,9 @@ impl ArgsKwargs {
     }
 
     pub fn __repr__(&self, py: Python) -> String {
-        let args = safe_repr(self.args.as_ref(py));
+        let args = safe_repr(self.args.bind(py));
         match self.kwargs {
-            Some(ref d) => format!("ArgsKwargs({args}, {})", safe_repr(d.as_ref(py))),
+            Some(ref d) => format!("ArgsKwargs({args}, {})", safe_repr(d.bind(py))),
             None => format!("ArgsKwargs({args})"),
         }
     }
@@ -94,7 +94,7 @@ impl PydanticUndefinedType {
     }
 
     #[pyo3(signature = (_memo, /))]
-    fn __deepcopy__(&self, py: Python, _memo: &PyAny) -> Py<Self> {
+    fn __deepcopy__(&self, py: Python, _memo: &Bound<'_, PyAny>) -> Py<Self> {
         self.__copy__(py)
     }
 
