@@ -21,28 +21,21 @@ impl BuildValidator for IsInstanceValidator {
     const EXPECTED_TYPE: &'static str = "is-instance";
 
     fn build(
-        schema: &PyDict,
-        _config: Option<&PyDict>,
+        schema: &Bound<'_, PyDict>,
+        _config: Option<&Bound<'_, PyDict>>,
         _definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
         let cls_key = intern!(py, "cls");
-        let class: &PyAny = schema.get_as_req(cls_key)?;
+        let class = schema.get_as_req(cls_key)?;
 
         // test that class works with isinstance to avoid errors at call time, reuse cls_key since it doesn't
         // matter what object is being checked
-        let test_value: &PyAny = cls_key.as_ref();
-        if test_value.is_instance(class).is_err() {
+        if cls_key.is_instance(&class).is_err() {
             return py_schema_err!("'cls' must be valid as the first argument to 'isinstance'");
         }
 
-        let class_repr = match schema.get_as(intern!(py, "cls_repr"))? {
-            Some(s) => s,
-            None => match class.extract::<&PyType>() {
-                Ok(t) => t.name()?.to_string(),
-                Err(_) => class.repr()?.extract()?,
-            },
-        };
+        let class_repr = class_repr(schema, &class)?;
         let name = format!("{}[{class_repr}]", Self::EXPECTED_TYPE);
         Ok(Self {
             class: class.into(),
@@ -56,22 +49,20 @@ impl BuildValidator for IsInstanceValidator {
 impl_py_gc_traverse!(IsInstanceValidator { class });
 
 impl Validator for IsInstanceValidator {
-    fn validate<'data>(
+    fn validate<'py>(
         &self,
-        py: Python<'data>,
-        input: &'data impl Input<'data>,
-        _state: &mut ValidationState,
-    ) -> ValResult<'data, PyObject> {
-        if !input.is_python() {
+        py: Python<'py>,
+        input: &(impl Input<'py> + ?Sized),
+        _state: &mut ValidationState<'_, 'py>,
+    ) -> ValResult<PyObject> {
+        let Some(obj) = input.as_python() else {
             return Err(ValError::InternalErr(PyNotImplementedError::new_err(
                 "Cannot check isinstance when validating from json, \
                             use a JsonOrPython validator instead.",
             )));
-        }
-
-        let ob = input.to_object(py);
-        match ob.as_ref(py).is_instance(self.class.as_ref(py))? {
-            true => Ok(ob),
+        };
+        match obj.is_instance(self.class.bind(py))? {
+            true => Ok(obj.clone().unbind()),
             false => Err(ValError::new(
                 ErrorType::IsInstanceOf {
                     class: self.class_repr.clone(),
@@ -86,3 +77,13 @@ impl Validator for IsInstanceValidator {
         &self.name
     }
 }
+
+pub fn class_repr(schema: &Bound<'_, PyDict>, class: &Bound<'_, PyAny>) -> PyResult<String> {
+    match schema.get_as(intern!(schema.py(), "cls_repr"))? {
+        Some(s) => Ok(s),
+        None => match class.extract::<&PyType>() {
+            Ok(t) => Ok(t.qualname()?.to_string()),
+            Err(_) => Ok(class.repr()?.extract()?),
+        },
+    }
+}
