@@ -4,9 +4,10 @@ use pyo3::prelude::*;
 use pyo3::types::PyDict;
 
 use crate::errors::ValResult;
-use crate::input::{GenericIterable, Input};
+use crate::input::{
+    no_validator_iter_to_vec, validate_iter_to_vec, BorrowInput, ConsumeIterator, Input, MaxLengthCheck, ValidatedList,
+};
 use crate::tools::SchemaDict;
-use crate::validators::Exactness;
 
 use super::{build_validator, BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationState, Validator};
 
@@ -20,13 +21,13 @@ pub struct ListValidator {
 }
 
 pub fn get_items_schema(
-    schema: &PyDict,
-    config: Option<&PyDict>,
+    schema: &Bound<'_, PyDict>,
+    config: Option<&Bound<'_, PyDict>>,
     definitions: &mut DefinitionsBuilder<CombinedValidator>,
 ) -> PyResult<Option<CombinedValidator>> {
     match schema.get_item(pyo3::intern!(schema.py(), "items_schema"))? {
         Some(d) => {
-            let validator = build_validator(d, config, definitions)?;
+            let validator = build_validator(&d, config, definitions)?;
             match validator {
                 CombinedValidator::Any(_) => Ok(None),
                 _ => Ok(Some(validator)),
@@ -96,8 +97,8 @@ impl BuildValidator for ListValidator {
     const EXPECTED_TYPE: &'static str = "list";
 
     fn build(
-        schema: &PyDict,
-        config: Option<&PyDict>,
+        schema: &Bound<'_, PyDict>,
+        config: Option<&Bound<'_, PyDict>>,
         definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
@@ -116,30 +117,40 @@ impl BuildValidator for ListValidator {
 impl_py_gc_traverse!(ListValidator { item_validator });
 
 impl Validator for ListValidator {
-    fn validate<'data>(
+    fn validate<'py>(
         &self,
-        py: Python<'data>,
-        input: &'data impl Input<'data>,
-        state: &mut ValidationState,
-    ) -> ValResult<'data, PyObject> {
-        let seq = input.validate_list(state.strict_or(self.strict))?;
-        let exactness = match &seq {
-            GenericIterable::List(_) | GenericIterable::JsonArray(_) => Exactness::Exact,
-            GenericIterable::Tuple(_) => Exactness::Strict,
-            _ => Exactness::Lax,
-        };
-        state.floor_exactness(exactness);
+        py: Python<'py>,
+        input: &(impl Input<'py> + ?Sized),
+        state: &mut ValidationState<'_, 'py>,
+    ) -> ValResult<PyObject> {
+        let seq = input.validate_list(state.strict_or(self.strict))?.unpack(state);
 
+        let actual_length = seq.len();
         let output = match self.item_validator {
-            Some(ref v) => seq.validate_to_vec(py, input, self.max_length, "List", v, state)?,
-            None => match seq {
-                GenericIterable::List(list) => {
-                    length_check!(input, "List", self.min_length, self.max_length, list);
-                    let list_copy = list.get_slice(0, usize::MAX);
+            Some(ref v) => seq.iterate(ValidateToVec {
+                py,
+                input,
+                actual_length,
+                max_length: self.max_length,
+                field_type: "List",
+                item_validator: v,
+                state,
+            })??,
+            None => {
+                if let Some(py_list) = seq.as_py_list() {
+                    length_check!(input, "List", self.min_length, self.max_length, py_list);
+                    let list_copy = py_list.get_slice(0, usize::MAX);
                     return Ok(list_copy.into_py(py));
                 }
-                _ => seq.to_vec(py, input, "List", self.max_length)?,
-            },
+
+                seq.iterate(ToVec {
+                    py,
+                    input,
+                    actual_length,
+                    max_length: self.max_length,
+                    field_type: "List",
+                })??
+            }
         };
         min_length_check!(input, "List", self.min_length, output);
         Ok(output.into_py(py))
@@ -164,3 +175,54 @@ impl Validator for ListValidator {
         }
     }
 }
+
+struct ValidateToVec<'a, 's, 'py, I: Input<'py> + ?Sized> {
+    py: Python<'py>,
+    input: &'a I,
+    actual_length: Option<usize>,
+    max_length: Option<usize>,
+    field_type: &'static str,
+    item_validator: &'a CombinedValidator,
+    state: &'a mut ValidationState<'s, 'py>,
+}
+
+// pretty arbitrary default capacity when creating vecs from iteration
+const DEFAULT_CAPACITY: usize = 10;
+
+impl<'py, T, I: Input<'py> + ?Sized> ConsumeIterator<PyResult<T>> for ValidateToVec<'_, '_, 'py, I>
+where
+    T: BorrowInput<'py>,
+{
+    type Output = ValResult<Vec<PyObject>>;
+    fn consume_iterator(self, iterator: impl Iterator<Item = PyResult<T>>) -> ValResult<Vec<PyObject>> {
+        let capacity = self.actual_length.unwrap_or(DEFAULT_CAPACITY);
+        let max_length_check = MaxLengthCheck::new(self.max_length, self.field_type, self.input, self.actual_length);
+        validate_iter_to_vec(
+            self.py,
+            iterator,
+            capacity,
+            max_length_check,
+            self.item_validator,
+            self.state,
+        )
+    }
+}
+
+struct ToVec<'a, 'py, I: Input<'py> + ?Sized> {
+    py: Python<'py>,
+    input: &'a I,
+    actual_length: Option<usize>,
+    max_length: Option<usize>,
+    field_type: &'static str,
+}
+
+impl<'py, T, I: Input<'py> + ?Sized> ConsumeIterator<PyResult<T>> for ToVec<'_, 'py, I>
+where
+    T: BorrowInput<'py>,
+{
+    type Output = ValResult<Vec<PyObject>>;
+    fn consume_iterator(self, iterator: impl Iterator<Item = PyResult<T>>) -> ValResult<Vec<PyObject>> {
+        let max_length_check = MaxLengthCheck::new(self.max_length, self.field_type, self.input, self.actual_length);
+        no_validator_iter_to_vec(self.py, self.input, iterator, max_length_check)
+    }
+}
