@@ -1,10 +1,9 @@
 use std::ptr::null_mut;
 
-use pyo3::conversion::AsPyPointer;
 use pyo3::exceptions::PyTypeError;
-use pyo3::prelude::*;
+use pyo3::ffi;
 use pyo3::types::{PyDict, PySet, PyString, PyTuple, PyType};
-use pyo3::{ffi, intern};
+use pyo3::{intern, prelude::*};
 
 use super::function::convert_err;
 use super::validation_state::Exactness;
@@ -14,7 +13,7 @@ use super::{
 use crate::build_tools::py_schema_err;
 use crate::build_tools::schema_or_config_same;
 use crate::errors::{ErrorType, ErrorTypeDefaults, ValError, ValResult};
-use crate::input::{py_error_on_minusone, Input};
+use crate::input::{input_as_python_instance, py_error_on_minusone, Input};
 use crate::tools::{py_err, SchemaDict};
 use crate::PydanticUndefinedType;
 
@@ -42,11 +41,11 @@ impl Revalidate {
         }
     }
 
-    pub fn should_revalidate(&self, input: &PyAny, class: &PyType) -> bool {
+    pub fn should_revalidate(&self, input: &Bound<'_, PyAny>, class: &Bound<'_, PyType>) -> bool {
         match self {
             Revalidate::Always => true,
             Revalidate::Never => false,
-            Revalidate::SubclassInstances => !input.get_type().is(class),
+            Revalidate::SubclassInstances => !input.is_exact_instance(class),
         }
     }
 }
@@ -68,36 +67,39 @@ impl BuildValidator for ModelValidator {
     const EXPECTED_TYPE: &'static str = "model";
 
     fn build(
-        schema: &PyDict,
-        _config: Option<&PyDict>,
+        schema: &Bound<'_, PyDict>,
+        _config: Option<&Bound<'_, PyDict>>,
         definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
         // models ignore the parent config and always use the config from this model
         let config = schema.get_as(intern!(py, "config"))?;
 
-        let class: &PyType = schema.get_as_req(intern!(py, "cls"))?;
-        let sub_schema: &PyAny = schema.get_as_req(intern!(py, "schema"))?;
-        let validator = build_validator(sub_schema, config, definitions)?;
+        let class: Bound<'_, PyType> = schema.get_as_req(intern!(py, "cls"))?;
+        let sub_schema = schema.get_as_req(intern!(py, "schema"))?;
+        let validator = build_validator(&sub_schema, config.as_ref(), definitions)?;
+        let name = class.getattr(intern!(py, "__name__"))?.extract()?;
 
         Ok(Self {
-            revalidate: Revalidate::from_str(schema_or_config_same(
-                schema,
-                config,
-                intern!(py, "revalidate_instances"),
-            )?)?,
+            revalidate: Revalidate::from_str(
+                schema_or_config_same::<Bound<'_, PyString>>(
+                    schema,
+                    config.as_ref(),
+                    intern!(py, "revalidate_instances"),
+                )?
+                .as_ref()
+                .map(|s| s.to_str())
+                .transpose()?,
+            )?,
             validator: Box::new(validator),
             class: class.into(),
-            post_init: schema
-                .get_as::<&str>(intern!(py, "post_init"))?
-                .map(|s| PyString::new(py, s).into_py(py)),
+            post_init: schema.get_as(intern!(py, "post_init"))?,
             frozen: schema.get_as(intern!(py, "frozen"))?.unwrap_or(false),
             custom_init: schema.get_as(intern!(py, "custom_init"))?.unwrap_or(false),
             root_model: schema.get_as(intern!(py, "root_model"))?.unwrap_or(false),
             undefined: PydanticUndefinedType::new(py).to_object(py),
-            // Get the class's `__name__`, not using `class.name()` since it uses `__qualname__`
-            // which is not what we want here
-            name: class.getattr(intern!(py, "__name__"))?.extract()?,
+            // Get the class's `__name__`, not using `class.qualname()`
+            name,
         }
         .into())
     }
@@ -106,11 +108,11 @@ impl BuildValidator for ModelValidator {
 impl_py_gc_traverse!(ModelValidator { class, validator });
 
 impl Validator for ModelValidator {
-    fn validate<'data>(
+    fn validate<'py>(
         &self,
-        py: Python<'data>,
-        input: &'data impl Input<'data>,
-        state: &mut ValidationState,
+        py: Python<'py>,
+        input: &(impl Input<'py> + ?Sized),
+        state: &mut ValidationState<'_, 'py>,
     ) -> ValResult<PyObject> {
         if let Some(self_instance) = state.extra().self_instance {
             // in the case that self_instance is Some, we're calling validation from within `BaseModel.__init__`
@@ -121,26 +123,26 @@ impl Validator for ModelValidator {
         // if we're not in strict mode, instances subclasses are okay, as well as dicts, mappings, from attributes etc.
         // if the input is an instance of the class, we "revalidate" it - e.g. we extract and reuse `__pydantic_fields_set__`
         // but use from attributes to create a new instance of the model field type
-        let class = self.class.as_ref(py);
-        if let Some(py_input) = input.input_is_instance(class) {
+        let class = self.class.bind(py);
+        if let Some(py_input) = input_as_python_instance(input, class) {
             if self.revalidate.should_revalidate(py_input, class) {
                 let fields_set = py_input.getattr(intern!(py, DUNDER_FIELDS_SET_KEY))?;
                 if self.root_model {
                     let inner_input = py_input.getattr(intern!(py, ROOT_FIELD))?;
-                    self.validate_construct(py, inner_input, Some(fields_set), state)
+                    self.validate_construct(py, &inner_input, Some(&fields_set), state)
                 } else {
                     // get dict here so from_attributes logic doesn't apply
                     let dict = py_input.getattr(intern!(py, DUNDER_DICT))?;
                     let model_extra = py_input.getattr(intern!(py, DUNDER_MODEL_EXTRA_KEY))?;
 
-                    let inner_input: &PyAny = if model_extra.is_none() {
+                    let inner_input = if PyAnyMethods::is_none(&model_extra) {
                         dict
                     } else {
                         let full_model_dict = dict.downcast::<PyDict>()?.copy()?;
                         full_model_dict.update(model_extra.downcast()?)?;
-                        full_model_dict
+                        full_model_dict.into_any()
                     };
-                    self.validate_construct(py, inner_input, Some(fields_set), state)
+                    self.validate_construct(py, &inner_input, Some(&fields_set), state)
                 }
             } else {
                 Ok(input.to_object(py))
@@ -152,13 +154,13 @@ impl Validator for ModelValidator {
         }
     }
 
-    fn validate_assignment<'data>(
+    fn validate_assignment<'py>(
         &self,
-        py: Python<'data>,
-        model: &'data PyAny,
-        field_name: &'data str,
-        field_value: &'data PyAny,
-        state: &mut ValidationState,
+        py: Python<'py>,
+        model: &Bound<'py, PyAny>,
+        field_name: &str,
+        field_value: &Bound<'py, PyAny>,
+        state: &mut ValidationState<'_, 'py>,
     ) -> ValResult<PyObject> {
         if self.frozen {
             return Err(ValError::new(ErrorTypeDefaults::FrozenInstance, field_value));
@@ -179,23 +181,26 @@ impl Validator for ModelValidator {
                 Ok(model.into_py(py))
             };
         }
-        let old_dict: &PyDict = model.getattr(intern!(py, DUNDER_DICT))?.downcast()?;
+        let old_dict = model.getattr(intern!(py, DUNDER_DICT))?.downcast_into::<PyDict>()?;
 
         let input_dict = old_dict.copy()?;
-        let old_extra: Option<&PyDict> = model.getattr(intern!(py, DUNDER_MODEL_EXTRA_KEY))?.downcast().ok();
-        if let Some(old_extra) = old_extra {
+        if let Ok(old_extra) = model.getattr(intern!(py, DUNDER_MODEL_EXTRA_KEY))?.downcast::<PyDict>() {
             input_dict.update(old_extra.as_mapping())?;
         }
         input_dict.set_item(field_name, field_value)?;
 
         let output = self
             .validator
-            .validate_assignment(py, input_dict, field_name, field_value, state)?;
+            .validate_assignment(py, input_dict.as_any(), field_name, field_value, state)?;
 
-        let (validated_dict, validated_extra, validated_fields_set): (&PyDict, &PyAny, &PySet) = output.extract(py)?;
+        let (validated_dict, validated_extra, validated_fields_set): (
+            Bound<'_, PyDict>,
+            Bound<'_, PyAny>,
+            Bound<'_, PySet>,
+        ) = output.extract(py)?;
 
         if let Ok(fields_set) = model.getattr(intern!(py, DUNDER_FIELDS_SET_KEY)) {
-            let fields_set: &PySet = fields_set.downcast()?;
+            let fields_set = fields_set.downcast::<PySet>()?;
             for field_name in validated_fields_set {
                 fields_set.add(field_name)?;
             }
@@ -218,12 +223,12 @@ impl Validator for ModelValidator {
 
 impl ModelValidator {
     /// here we just call the inner validator, then set attributes on `self_instance`
-    fn validate_init<'s, 'data>(
-        &'s self,
-        py: Python<'data>,
-        self_instance: &'s PyAny,
-        input: &'data impl Input<'data>,
-        state: &mut ValidationState,
+    fn validate_init<'py>(
+        &self,
+        py: Python<'py>,
+        self_instance: &Bound<'py, PyAny>,
+        input: &(impl Input<'py> + ?Sized),
+        state: &mut ValidationState<'_, 'py>,
     ) -> ValResult<PyObject> {
         // we need to set `self_instance` to None for nested validators as we don't want to operate on self_instance
         // anymore
@@ -232,25 +237,25 @@ impl ModelValidator {
 
         if self.root_model {
             let fields_set = if input.to_object(py).is(&self.undefined) {
-                PySet::empty(py)?
+                PySet::empty_bound(py)?
             } else {
-                PySet::new(py, [&String::from(ROOT_FIELD)])?
+                PySet::new_bound(py, [&String::from(ROOT_FIELD)])?
             };
             force_setattr(py, self_instance, intern!(py, DUNDER_FIELDS_SET_KEY), fields_set)?;
-            force_setattr(py, self_instance, intern!(py, ROOT_FIELD), output.as_ref(py))?;
+            force_setattr(py, self_instance, intern!(py, ROOT_FIELD), &output)?;
         } else {
-            let (model_dict, model_extra, fields_set): (&PyAny, &PyAny, &PyAny) = output.extract(py)?;
-            set_model_attrs(self_instance, model_dict, model_extra, fields_set)?;
+            let (model_dict, model_extra, fields_set) = output.extract(py)?;
+            set_model_attrs(self_instance, &model_dict, &model_extra, &fields_set)?;
         }
-        self.call_post_init(py, self_instance.into_py(py), input, state.extra())
+        self.call_post_init(py, self_instance.clone(), input, state.extra())
     }
 
-    fn validate_construct<'s, 'data>(
-        &'s self,
-        py: Python<'data>,
-        input: &'data impl Input<'data>,
-        existing_fields_set: Option<&'data PyAny>,
-        state: &mut ValidationState,
+    fn validate_construct<'py>(
+        &self,
+        py: Python<'py>,
+        input: &(impl Input<'py> + ?Sized),
+        existing_fields_set: Option<&Bound<'_, PyAny>>,
+        state: &mut ValidationState<'_, 'py>,
     ) -> ValResult<PyObject> {
         if self.custom_init {
             // If we wanted, we could introspect the __init__ signature, and store the
@@ -261,59 +266,58 @@ impl ModelValidator {
             if let Some(kwargs) = input.as_kwargs(py) {
                 return self
                     .class
-                    .call(py, (), Some(kwargs))
+                    .call_bound(py, (), Some(&kwargs))
                     .map_err(|e| convert_err(py, e, input));
             }
         }
 
         let output = self.validator.validate(py, input, state)?;
 
-        let instance = create_class(self.class.as_ref(py))?;
-        let instance_ref = instance.as_ref(py);
+        let instance = create_class(self.class.bind(py))?;
 
         if self.root_model {
             let fields_set = if input.to_object(py).is(&self.undefined) {
-                PySet::empty(py)?
+                PySet::empty_bound(py)?
             } else {
-                PySet::new(py, [&String::from(ROOT_FIELD)])?
+                PySet::new_bound(py, [&String::from(ROOT_FIELD)])?
             };
-            force_setattr(py, instance_ref, intern!(py, DUNDER_FIELDS_SET_KEY), fields_set)?;
-            force_setattr(py, instance_ref, intern!(py, ROOT_FIELD), output)?;
+            force_setattr(py, &instance, intern!(py, DUNDER_FIELDS_SET_KEY), fields_set)?;
+            force_setattr(py, &instance, intern!(py, ROOT_FIELD), output)?;
         } else {
-            let (model_dict, model_extra, val_fields_set): (&PyAny, &PyAny, &PyAny) = output.extract(py)?;
-            let fields_set = existing_fields_set.unwrap_or(val_fields_set);
-            set_model_attrs(instance_ref, model_dict, model_extra, fields_set)?;
+            let (model_dict, model_extra, val_fields_set) = output.extract(py)?;
+            let fields_set = existing_fields_set.unwrap_or(&val_fields_set);
+            set_model_attrs(&instance, &model_dict, &model_extra, fields_set)?;
         }
         self.call_post_init(py, instance, input, state.extra())
     }
 
-    fn call_post_init<'s, 'data>(
-        &'s self,
-        py: Python<'data>,
-        instance: PyObject,
-        input: &'data impl Input<'data>,
+    fn call_post_init<'py>(
+        &self,
+        py: Python<'py>,
+        instance: Bound<'_, PyAny>,
+        input: &(impl Input<'py> + ?Sized),
         extra: &Extra,
     ) -> ValResult<PyObject> {
         if let Some(ref post_init) = self.post_init {
             instance
-                .call_method1(py, post_init.as_ref(py), (extra.context,))
+                .call_method1(post_init.bind(py), (extra.context,))
                 .map_err(|e| convert_err(py, e, input))?;
         }
-        Ok(instance)
+        Ok(instance.into())
     }
 }
 
 /// based on the following but with the second argument of new_func set to an empty tuple as required
 /// https://github.com/PyO3/pyo3/blob/d2caa056e9aacc46374139ef491d112cb8af1a25/src/pyclass_init.rs#L35-L77
-pub(super) fn create_class(class: &PyType) -> PyResult<PyObject> {
+pub(super) fn create_class<'py>(class: &Bound<'py, PyType>) -> PyResult<Bound<'py, PyAny>> {
     let py = class.py();
-    let args = PyTuple::empty(py);
+    let args = PyTuple::empty_bound(py);
     let raw_type = class.as_type_ptr();
     unsafe {
         // Safety: raw_type is known to be a non-null type object pointer
         match (*raw_type).tp_new {
             // Safety: the result of new_func is guaranteed to be either an owned pointer or null on error returns.
-            Some(new_func) => PyObject::from_owned_ptr_or_err(
+            Some(new_func) => Bound::from_owned_ptr_or_err(
                 py,
                 // Safety: the non-null pointers are known to be valid, and it's allowed to call tp_new with a
                 // null kwargs dict.
@@ -324,7 +328,12 @@ pub(super) fn create_class(class: &PyType) -> PyResult<PyObject> {
     }
 }
 
-fn set_model_attrs(instance: &PyAny, model_dict: &PyAny, model_extra: &PyAny, fields_set: &PyAny) -> PyResult<()> {
+fn set_model_attrs(
+    instance: &Bound<'_, PyAny>,
+    model_dict: &Bound<'_, PyAny>,
+    model_extra: &Bound<'_, PyAny>,
+    fields_set: &Bound<'_, PyAny>,
+) -> PyResult<()> {
     let py = instance.py();
     force_setattr(py, instance, intern!(py, DUNDER_DICT), model_dict)?;
     force_setattr(py, instance, intern!(py, DUNDER_MODEL_EXTRA_KEY), model_extra)?;
@@ -333,7 +342,7 @@ fn set_model_attrs(instance: &PyAny, model_dict: &PyAny, model_extra: &PyAny, fi
     Ok(())
 }
 
-pub(super) fn force_setattr<N, V>(py: Python<'_>, obj: &PyAny, attr_name: N, value: V) -> PyResult<()>
+pub(super) fn force_setattr<N, V>(py: Python<'_>, obj: &Bound<'_, PyAny>, attr_name: N, value: V) -> PyResult<()>
 where
     N: ToPyObject,
     V: ToPyObject,
