@@ -5,9 +5,9 @@ use std::collections::VecDeque;
 
 use crate::build_tools::is_strict;
 use crate::errors::{py_err_string, ErrorType, ErrorTypeDefaults, ValError, ValLineError, ValResult};
-use crate::input::{GenericIterable, Input};
+use crate::input::ConsumeIterator;
+use crate::input::{BorrowInput, Input, ValidatedTuple};
 use crate::tools::SchemaDict;
-use crate::validators::Exactness;
 
 use super::{build_validator, BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationState, Validator};
 
@@ -24,15 +24,15 @@ pub struct TupleValidator {
 impl BuildValidator for TupleValidator {
     const EXPECTED_TYPE: &'static str = "tuple";
     fn build(
-        schema: &PyDict,
-        config: Option<&PyDict>,
+        schema: &Bound<'_, PyDict>,
+        config: Option<&Bound<'_, PyDict>>,
         definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
-        let items: &PyList = schema.get_as_req(intern!(py, "items_schema"))?;
+        let items: Bound<'_, PyList> = schema.get_as_req(intern!(py, "items_schema"))?;
         let validators: Vec<CombinedValidator> = items
             .iter()
-            .map(|item| build_validator(item, config, definitions))
+            .map(|item| build_validator(&item, config, definitions))
             .collect::<PyResult<_>>()?;
 
         let mut validator_names = validators.iter().map(Validator::get_name).collect::<Vec<_>>();
@@ -59,24 +59,24 @@ impl_py_gc_traverse!(TupleValidator { validators });
 
 impl TupleValidator {
     #[allow(clippy::too_many_arguments)]
-    fn validate_tuple_items<'s, 'data, I: Input<'data> + 'data>(
+    fn validate_tuple_items<'py, I: BorrowInput<'py>>(
         &self,
-        py: Python<'data>,
-        input: &'data impl Input<'data>,
-        state: &mut ValidationState,
+        py: Python<'py>,
+        input: &(impl Input<'py> + ?Sized),
+        state: &mut ValidationState<'_, 'py>,
         output: &mut Vec<PyObject>,
         errors: &mut Vec<ValLineError>,
         item_validators: &[CombinedValidator],
-        collection_iter: &mut NextCountingIterator<impl Iterator<Item = &'data I>>,
+        collection_iter: &mut NextCountingIterator<impl Iterator<Item = I>>,
         actual_length: Option<usize>,
     ) -> ValResult<()> {
         // Validate the head:
         for validator in item_validators {
             match collection_iter.next() {
-                Some((index, input_item)) => match validator.validate(py, input_item, state) {
+                Some((index, input_item)) => match validator.validate(py, input_item.borrow_input(), state) {
                     Ok(item) => self.push_output_item(input, output, item, actual_length)?,
                     Err(ValError::LineErrors(line_errors)) => {
-                        errors.extend(line_errors.into_iter().map(|err| err.with_outer_location(index.into())));
+                        errors.extend(line_errors.into_iter().map(|err| err.with_outer_location(index)));
                     }
                     Err(ValError::Omit) => (),
                     Err(err) => return Err(err),
@@ -96,13 +96,13 @@ impl TupleValidator {
     }
 
     #[allow(clippy::too_many_arguments)]
-    fn validate_tuple_variable<'data, I: Input<'data> + 'data, InputT: Input<'data> + 'data>(
+    fn validate_tuple_variable<'py, I: BorrowInput<'py>>(
         &self,
-        py: Python<'data>,
-        input: &'data InputT,
-        state: &mut ValidationState,
+        py: Python<'py>,
+        input: &(impl Input<'py> + ?Sized),
+        state: &mut ValidationState<'_, 'py>,
         errors: &mut Vec<ValLineError>,
-        collection_iter: &mut NextCountingIterator<impl Iterator<Item = &'data I>>,
+        collection_iter: &mut NextCountingIterator<impl Iterator<Item = I>>,
         actual_length: Option<usize>,
     ) -> ValResult<Vec<PyObject>> {
         let expected_length = if self.variadic_item_index.is_some() {
@@ -133,10 +133,10 @@ impl TupleValidator {
             let n_tail_validators = tail_validators.len();
             if n_tail_validators == 0 {
                 for (index, input_item) in collection_iter {
-                    match variable_validator.validate(py, input_item, state) {
+                    match variable_validator.validate(py, input_item.borrow_input(), state) {
                         Ok(item) => self.push_output_item(input, &mut output, item, actual_length)?,
                         Err(ValError::LineErrors(line_errors)) => {
-                            errors.extend(line_errors.into_iter().map(|err| err.with_outer_location(index.into())));
+                            errors.extend(line_errors.into_iter().map(|err| err.with_outer_location(index)));
                         }
                         Err(ValError::Omit) => (),
                         Err(err) => return Err(err),
@@ -147,8 +147,7 @@ impl TupleValidator {
                 // NB: We take from collection_iter.inner to avoid increasing the next calls count
                 // while populating the buffer. This means the index in the following loop is the
                 // right one for user errors.
-                let mut tail_buffer: VecDeque<&'data I> =
-                    collection_iter.inner.by_ref().take(n_tail_validators).collect();
+                let mut tail_buffer: VecDeque<I> = collection_iter.inner.by_ref().take(n_tail_validators).collect();
 
                 // Save the current index for the tail validation below when we recreate a new NextCountingIterator
                 let mut index = collection_iter.next_calls();
@@ -161,13 +160,13 @@ impl TupleValidator {
                     let buffered_item = tail_buffer.pop_front().unwrap();
                     tail_buffer.push_back(input_item);
 
-                    match variable_validator.validate(py, buffered_item, state) {
+                    match variable_validator.validate(py, buffered_item.borrow_input(), state) {
                         Ok(item) => self.push_output_item(input, &mut output, item, actual_length)?,
                         Err(ValError::LineErrors(line_errors)) => {
                             errors.extend(
                                 line_errors
                                     .into_iter()
-                                    .map(|err| err.with_outer_location(buffer_item_index.into())),
+                                    .map(|err| err.with_outer_location(buffer_item_index)),
                             );
                         }
                         Err(ValError::Omit) => (),
@@ -216,9 +215,9 @@ impl TupleValidator {
         Ok(output)
     }
 
-    fn push_output_item<'data>(
+    fn push_output_item<'py>(
         &self,
-        input: &'data impl Input<'data>,
+        input: &(impl Input<'py> + ?Sized),
         output: &mut Vec<PyObject>,
         item: PyObject,
         actual_length: Option<usize>,
@@ -242,73 +241,25 @@ impl TupleValidator {
 }
 
 impl Validator for TupleValidator {
-    fn validate<'data>(
+    fn validate<'py>(
         &self,
-        py: Python<'data>,
-        input: &'data impl Input<'data>,
-        state: &mut ValidationState,
+        py: Python<'py>,
+        input: &(impl Input<'py> + ?Sized),
+        state: &mut ValidationState<'_, 'py>,
     ) -> ValResult<PyObject> {
-        let collection: GenericIterable<'_> = input.validate_tuple(state.strict_or(self.strict))?;
-        let exactness = match &collection {
-            GenericIterable::Tuple(_) | GenericIterable::JsonArray(_) => Exactness::Exact,
-            GenericIterable::List(_) => Exactness::Strict,
-            _ => Exactness::Lax,
-        };
-        state.floor_exactness(exactness);
-        let actual_length = collection.generic_len();
+        let collection = input.validate_tuple(state.strict_or(self.strict))?.unpack(state);
+        let actual_length = collection.len();
 
         let mut errors: Vec<ValLineError> = Vec::new();
 
-        let mut iteration_error = None;
-
-        macro_rules! iter {
-            ($collection_iter:expr) => {
-                self.validate_tuple_variable(
-                    py,
-                    input,
-                    state,
-                    &mut errors,
-                    &mut NextCountingIterator::new($collection_iter, 0),
-                    actual_length,
-                )
-            };
-        }
-
-        let output = match collection {
-            GenericIterable::List(collection_iter) => iter!(collection_iter.iter())?,
-            GenericIterable::Tuple(collection_iter) => iter!(collection_iter.iter())?,
-            GenericIterable::JsonArray(collection_iter) => iter!(collection_iter.iter())?,
-            other => iter!({
-                let mut sequence_iterator = other.as_sequence_iterator(py)?;
-                let iteration_error = &mut iteration_error;
-                let mut index: usize = 0;
-                std::iter::from_fn(move || {
-                    if iteration_error.is_some() {
-                        return None;
-                    }
-                    index += 1;
-                    match sequence_iterator.next() {
-                        Some(Ok(item)) => Some(item),
-                        Some(Err(e)) => {
-                            *iteration_error = Some(ValError::new_with_loc(
-                                ErrorType::IterationError {
-                                    error: py_err_string(py, e),
-                                    context: None,
-                                },
-                                input,
-                                index,
-                            ));
-                            None
-                        }
-                        None => None,
-                    }
-                })
-            })?,
-        };
-
-        if let Some(err) = iteration_error {
-            return Err(err);
-        }
+        let output = collection.iterate(ValidateToTuple {
+            py,
+            input,
+            actual_length,
+            validator: self,
+            errors: &mut errors,
+            state,
+        })??;
 
         if let Some(min_length) = self.min_length {
             let actual_length = output.len();
@@ -326,7 +277,7 @@ impl Validator for TupleValidator {
         }
 
         if errors.is_empty() {
-            Ok(PyTuple::new(py, &output).into_py(py))
+            Ok(PyTuple::new_bound(py, output).into_py(py))
         } else {
             Err(ValError::LineErrors(errors))
         }
@@ -337,6 +288,68 @@ impl Validator for TupleValidator {
     }
 }
 
+struct ValidateToTuple<'a, 's, 'py, I: Input<'py> + ?Sized> {
+    py: Python<'py>,
+    input: &'a I,
+    actual_length: Option<usize>,
+    validator: &'a TupleValidator,
+    errors: &'a mut Vec<ValLineError>,
+    state: &'a mut ValidationState<'s, 'py>,
+}
+
+impl<'py, T, I> ConsumeIterator<PyResult<T>> for ValidateToTuple<'_, '_, 'py, I>
+where
+    T: BorrowInput<'py>,
+    I: Input<'py> + ?Sized,
+{
+    type Output = ValResult<Vec<PyObject>>;
+    fn consume_iterator(self, mut iterator: impl Iterator<Item = PyResult<T>>) -> ValResult<Vec<PyObject>> {
+        let mut iteration_error = None;
+
+        let output = self.validator.validate_tuple_variable(
+            self.py,
+            self.input,
+            self.state,
+            self.errors,
+            &mut NextCountingIterator::new(
+                {
+                    let iteration_error = &mut iteration_error;
+                    let mut index: usize = 0;
+                    std::iter::from_fn(move || {
+                        if iteration_error.is_some() {
+                            return None;
+                        }
+                        index += 1;
+                        match iterator.next() {
+                            Some(Ok(item)) => Some(item),
+                            Some(Err(e)) => {
+                                *iteration_error = Some(ValError::new_with_loc(
+                                    ErrorType::IterationError {
+                                        error: py_err_string(self.py, e),
+                                        context: None,
+                                    },
+                                    self.input,
+                                    index,
+                                ));
+                                None
+                            }
+                            None => None,
+                        }
+                    })
+                },
+                0,
+            ),
+            self.actual_length,
+        )?;
+
+        if let Some(err) = iteration_error {
+            return Err(err);
+        }
+
+        Ok(output)
+    }
+}
+
 struct NextCountingIterator<I: Iterator> {
     inner: I,
     count: usize,
