@@ -1,11 +1,13 @@
 import dataclasses
 import json
 import platform
+import re
 import sys
 from collections import namedtuple
 from datetime import date, datetime, time, timedelta, timezone
 from decimal import Decimal
 from enum import Enum
+from math import inf, isinf, isnan, nan
 from pathlib import Path
 from typing import ClassVar
 
@@ -104,7 +106,7 @@ def test_set_member_db(any_serializer):
         (datetime(2032, 1, 1, 1, 1), b'"2032-01-01T01:01:00"'),
         (date(2022, 12, 3), b'"2022-12-03"'),
         (time(12, 30, 45), b'"12:30:45"'),
-        (timedelta(hours=2), b'"PT7200S"'),
+        (timedelta(hours=2), b'"PT2H"'),
         (MyDataclass(1, 'foo', 2), b'{"a":1,"b":"foo"}'),
         (MyModel(a=1, b='foo'), b'{"a":1,"b":"foo"}'),
         ([MyDataclass(1, 'a', 2), MyModel(a=2, b='b')], b'[{"a":1,"b":"a"},{"a":2,"b":"b"}]'),
@@ -163,8 +165,8 @@ def test_any_with_date_serializer():
 def test_any_with_timedelta_serializer():
     s = SchemaSerializer(core_schema.any_schema(serialization={'type': 'timedelta'}))
     assert s.to_python(timedelta(hours=2)) == timedelta(hours=2)
-    assert s.to_python(timedelta(hours=2), mode='json') == 'PT7200S'
-    assert s.to_json(timedelta(hours=2)) == b'"PT7200S"'
+    assert s.to_python(timedelta(hours=2), mode='json') == 'PT2H'
+    assert s.to_json(timedelta(hours=2)) == b'"PT2H"'
 
     with pytest.warns(UserWarning) as warning_info:
         assert s.to_python(b'bang', mode='json') == 'bang'
@@ -437,7 +439,7 @@ def test_base64():
         (lambda: MyEnum.a, {}, b'1'),
         (lambda: MyEnum.b, {}, b'"b"'),
         (lambda: [MyDataclass(1, 'a', 2), MyModel(a=2, b='b')], {}, b'[{"a":1,"b":"a"},{"a":2,"b":"b"}]'),
-        # # (lambda: re.compile('^regex$'), b'"^regex$"'),
+        (lambda: re.compile('^regex$'), {}, b'"^regex$"'),
     ],
 )
 def test_encoding(any_serializer, gen_input, kwargs, expected_json):
@@ -602,3 +604,51 @@ def test_numpy_float(any_serializer):
         any_serializer.to_python(numpy.float16(1.0), mode='json')
     with pytest.raises(PydanticSerializationError, match=r"Unable to serialize unknown type: <class 'numpy\.float16'>"):
         any_serializer.to_json(numpy.float16(1.0))
+
+
+def test_ser_json_inf_nan_with_any() -> None:
+    s = SchemaSerializer(core_schema.any_schema(), core_schema.CoreConfig(ser_json_inf_nan='constants'))
+    assert isinf(s.to_python(inf))
+    assert isinf(s.to_python(inf, mode='json'))
+    assert s.to_json(inf) == b'Infinity'
+    assert isnan(s.to_python(nan))
+    assert isnan(s.to_python(nan, mode='json'))
+    assert s.to_json(nan) == b'NaN'
+
+    s = SchemaSerializer(core_schema.any_schema(), core_schema.CoreConfig(ser_json_inf_nan='null'))
+    assert isinf(s.to_python(inf))
+    assert s.to_python(inf, mode='json') is None
+    assert s.to_json(inf) == b'null'
+    assert isnan(s.to_python(nan))
+    assert s.to_python(nan, mode='json') is None
+    assert s.to_json(nan) == b'null'
+
+    s = SchemaSerializer(core_schema.any_schema(), core_schema.CoreConfig(ser_json_inf_nan='strings'))
+    assert isinf(s.to_python(inf))
+    assert isinf(s.to_python(inf, mode='json'))
+    assert s.to_json(inf) == b'"Infinity"'
+    assert isnan(s.to_python(nan))
+    assert isnan(s.to_python(nan, mode='json'))
+    assert s.to_json(nan) == b'"NaN"'
+
+
+def test_ser_json_inf_nan_with_list_of_any() -> None:
+    s = SchemaSerializer(
+        core_schema.list_schema(core_schema.any_schema()), core_schema.CoreConfig(ser_json_inf_nan='constants')
+    )
+    assert isinf(s.to_python([inf])[0])
+    assert isinf(s.to_python([inf], mode='json')[0])
+    assert s.to_json([inf]) == b'[Infinity]'
+    assert isnan(s.to_python([nan])[0])
+    assert isnan(s.to_python([nan], mode='json')[0])
+    assert s.to_json([nan]) == b'[NaN]'
+
+    s = SchemaSerializer(
+        core_schema.list_schema(core_schema.any_schema()), core_schema.CoreConfig(ser_json_inf_nan='null')
+    )
+    assert isinf(s.to_python([inf])[0])
+    assert s.to_python([inf], mode='json')[0] is None
+    assert s.to_json([inf]) == b'[null]'
+    assert isnan(s.to_python([nan])[0])
+    assert s.to_python([nan], mode='json')[0] is None
+    assert s.to_json([nan]) == b'[null]'
